/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Nexus/firewall/FirewallRepositoryContrib', ['Nexus/siesta'], function() {

  FirewallRepositoryContrib = function(config) {
    var me = this;
    me.reset = function(config) {
      var config = config || {};
      var defaultConfig = {
        grid: null
      };
      me.summaryData = {};
      me.summaryDataError = {};

      Ext.apply(me, config, defaultConfig);

      if (me.grid) {
        function getRepoSummary(record) {
          delete me.summaryData[record.data.id];
          delete me.summaryDataError[record.data.id];
          if (!me.isProxyRepository(record)) {
            me.summaryDataError[record.data.id] = '';
            me.grid.getView().refreshRow(record);
          }
          else {
            Ext.Ajax.request({
              url: Nexus.clm.firewallSummaryResourcePath + '/' + encodeURIComponent(record.data.id),
              method: 'GET',
              scope: self,
              suppressStatus: [400, 404, 409],
              success: function(response) {
                me.summaryData[record.data.id] = Ext.decode(response.responseText);
                me.grid.getView().refreshRow(record);
              },
              failure: function(response, opts) {
                var errorText = 'Unable to retrieve summary data';
                //400 means firewall isn't configured, so just leave column empty
                if (response.status === 400) {
                  errorText = '';
                }
                else if (response.status === 409) {
                  me.summaryData[record.data.id] = response.responseText;
                  errorText = '';
                }
                else if (response.siestaError) {
                  errorText = response.siestaError.message;
                }
                me.summaryDataError[record.data.id] = errorText;
                me.grid.getView().refreshRow(record);
              }
            });
          }
        }

        me.grid.store.addListener('load', function(store, records, options) {
          Ext.each(records, function(record) {
            getRepoSummary(record);
          });
        });

        me.grid.store.addListener('add', function(store, records, index) {
          Ext.each(records, function(record) {
            getRepoSummary(record);
          });
        });
      }
    };

    me.reset(config);

    me.isProxyRepository = function(record) {
      return record.get('repoType') == 'proxy';
    };

    me.updateColumns = function(columns) {
      columns.splice(8, 0, {
        header: 'IQ Policy Violations',
        id: 'firewallColumn',
        width: 175,
        renderer: function(value, metadata, record, rowIndex, colIndex, store) {
          if (me.summaryDataError[record.data.id]) {
            me.updateTooltip();
            return '<img class="nx-firewall-error-icon" src="' + Sonatype.config.resourcePath +
                '/static/images/exclamation.gif"><span class="nx-firewall-error">' +
                me.summaryDataError[record.data.id] + '</span>';
          }
          var summaryData = me.summaryData[record.data.id];

          if (summaryData) {
            if (typeof summaryData === 'string') {
              return Ext.util.Format.htmlEncode(summaryData);
            }

            function getChiclet(cssClass, count) {
              return '<div class="nx-firewall-chiclet ' + cssClass + '">' + count + '</div>';
            }

            var html = '<div class="nx-firewall-container">';
            if (summaryData.criticalComponentCount) {
              html += getChiclet('critical', summaryData.criticalComponentCount);
            }
            if (summaryData.severeComponentCount) {
              html += getChiclet('severe', summaryData.severeComponentCount);
            }
            if (summaryData.moderateComponentCount) {
              html += getChiclet('moderate', summaryData.moderateComponentCount);
            }
            if (!summaryData.criticalComponentCount && !summaryData.severeComponentCount && !summaryData.moderateComponentCount) {
              html += '<div style="float:left;">No violations</div>';
            }
            if (summaryData.quarantinedComponentCount) {
              html += '<div class="nx-firewall-quarantine-icon"></div><div class="nx-firewall-quarantine-count">' + summaryData.quarantinedComponentCount + '</div>';
            }
            html += '<div class="nx-external-link"></div>';
            if (summaryData.errorMessage) {
              me.updateTooltip();
              html += '<div class="nx-firewall-iq-error-icon"></div>';
              html += '<div class="nx-firewall-error">' + Ext.util.Format.htmlEncode(summaryData.errorMessage) + '</div>';
            }
            html += '</div>';

            return html;
          }
          return '';
        },
        clickHandler: function(grid, rowIndex, eventObject) {
          var record = me.grid.getStore().getAt(rowIndex);
          var summaryData = me.summaryData[record.data.id];
          if (summaryData && typeof summaryData === 'object') {
            window.open(summaryData.reportUrl, '_blank');
          }
        }
      });

      me.updateTooltip = function() {
        var view = me.grid.getView();
        if (me.grid.tip) {
          me.grid.tip.destroy();
        }
        me.grid.tip = new Ext.ToolTip({
          target : view.mainBody,
          delegate : '.x-grid3-cell',
          maxWidth : 500,
          mouseOffset : [ 0, 0 ],
          renderTo : document.body,
          listeners : {
            beforeshow : {
              fn : me.tooltipBeforeShowListener
            }
          }
        });
      };

      me.tooltipBeforeShowListener = function(tip) {
        tip.hideDelay = 200;
        var view = me.grid.getView(),
            rowIndex = view.findRowIndex(tip.triggerElement),
            cellIndex = view.findCellIndex(tip.triggerElement),
            record = me.grid.store.getAt(rowIndex);

        if (!record || cellIndex != me.grid.getColumnModel().getIndexById('firewallColumn')) {
          return false;
        }

        var tooltipText = me.summaryDataError[record.data.id];
        if (!tooltipText && me.summaryData[record.data.id]) {
          tooltipText = me.summaryData[record.data.id].errorMessage;
        }

        if (Ext.isEmpty(tooltipText)) {
          return false;
        }

        tip.body.dom.innerHTML = Ext.util.Format.htmlEncode(tooltipText);
      };
    };

    Sonatype.Events.addListener('repositoryPanelInit', function(repoPanel) {
      me.reset({
        grid: repoPanel.gridPanel
      });
      return true;
    });

    Sonatype.Events.addListener('repositoryGridInit', function(repoPanel, columns) {
      me.updateColumns(columns);
      return true;
    });
  };

  Sonatype.FirewallRepositoryContrib = new FirewallRepositoryContrib();
});
