/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Window presented to user to confirm an operation on a staging repository.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.ConfirmWindow', {
    extend: 'Ext.Window',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.staging.Icons'
    ],

    statics: {
        /**
         * @private
         */
        messageTpl: NX.create('Ext.Template',
            '<div class="nx-staging-repository-ConfirmWindow-messagePanel-box">',
                '<div class="nx-staging-repository-ConfirmWindow-messagePanel-icon">{icon}</div>',
                '<div class="nx-staging-repository-ConfirmWindow-messagePanel-text">{message}</div>',
            '</div>',
            {
                compiled: true,
                disableFormats: true
            })
    },

    /**
     * URL to submit to on confirmation.
     *
     * @type {String}
     */
    url: undefined,

    /**
     * Activity name
     *
     * @type {String}
     */
    activity: undefined,

    /**
     * Confirmation message.
     *
     * @type {String}
     */
    message: undefined,

    /**
     * @override
     */
    initComponent: function () {
        var self = this,
            icons = Nexus.staging.Icons,
            title;

        NX.assert(self.url !== undefined, 'Missing: url');
        NX.assert(self.activity !== undefined, 'Missing: activity');
        NX.assert(self.message !== undefined, 'Missing: message');

        title = Ext.util.Format.capitalize(self.activity) + ' Confirmation';

        /**
         * @private
         */
        self.messagePanel = NX.create('Ext.Panel', {
            cls: 'nx-staging-repository-ConfirmWindow-messagePanel',
            border: false, // custom border handling in css

            html: Nexus.staging.repository.ConfirmWindow.messageTpl.apply({
                icon: icons.forActivityName(self.activity).variant('x32').img, // use larger 32x32 icons if we have them
                message: self.message
            })
        });

        /**
         * @private
         */
        self.formPanel = NX.create('Ext.form.FormPanel', {
            cls: 'nx-staging-repository-ConfirmWindow-formPanel',
            border: false, // custom border handling in css
            monitorValid: true,
            labelWidth: 120, // increase width to avoid wrapping promote/release option labels

            items: self.buildFormItems(),

            buttons: [
                {
                    text: 'Confirm',
                    formBind: true,
                    scope: self,
                    handler: self.handleConfirm
                },
                {
                    xtype: 'link-button',
                    text: 'Cancel',
                    formBind: false,
                    scope: self,
                    handler: self.handleCancel
                }
            ],

            keys: [
                {
                    // Close dialog on ESC
                    key: Ext.EventObject.ESC,
                    scope: self,
                    fn: self.handleCancel
                }
            ]
        });

        self.addEvents(
            /**
             * @event
             */
            'confirm',

            /**
             * @event
             */
            'cancel',

            /**
             * @event
             */
            'success',

            /**
             * @event
             */
            'failure'
        );

        Ext.apply(self, {
            cls: 'nx-staging-repository-ConfirmWindow',
            title: title,

            // FIXME: icon isn't properly aligned ATM, disable it for now its distracting
            //iconCls: icons.get('warning').cls,

            autoHeight: true,
            width: 400,
            border: false,

            modal: true,
            constrain: true,
            closable: false,
            resizable: false,

            items: [
                self.messagePanel,
                self.formPanel
            ],

            listeners: {
                show: function (window) {
                    // auto-focus comment/description if we are showing it
                    self.formPanel.find('name', 'description')[0].focus(false, 100);
                }
            }
        });

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * @private
     *
     * @return {Array}
     */
    buildFormItems: function () {
        var self = this,
            items = [];

        self.customizeFormItems(items);

        items.push({
            xtype: 'fieldset',
            title: 'Description',
            items: [
                {
                    name: 'description',
                    xtype: 'textarea',
                    height: 50,
                    anchor: '100%',
                    hideLabel: true

                    // Disable for now, as we auto-focus this field and its distracting to see this flicker off once the focus is changed
                    //emptyText: 'Optional description or comment'
                }
            ]
        });

        return items;
    },

    /**
     * Customize form items.
     *
     * @protected
     *
     * @param {Array} items
     */
    customizeFormItems: function (items) {
        // override to customize
    },

    /**
     * @private
     */
    handleConfirm: function () {
        var self = this,
            form,
            data,
            mask;

        self.fireEvent('confirm', self);

        form = self.formPanel.getForm();
        data = form.getValues();
        data = self.customizeFormData(form, data)

        mask = NX.create('Ext.LoadMask', self.body, {
            msg: 'Please wait...'
        });
        mask.show();

        Ext.Ajax.request({
            method: 'POST',
            url: self.url,

            options: {
                dontForceLogout: true
            },

            jsonData: data,

            scope: self,
            success: function (response, options) {
                self.handleSuccess(response, options);
                mask.hide();
            },
            failure: function (response, options) {
                self.handleFailure(response, options);
                mask.hide();
            }
        });
    },

    /**
     * Customize form data to submit.
     *
     * @protected
     *
     * @param {Ext.form.BasicForm} form     Form which collected details.
     * @param {*} values                    The form's current values.
     * @return {*}                          Form data to submit.
     */
    customizeFormData: function(form, values) {
        // override to customize
        return values;
    },

    /**
     * @private
     */
    handleCancel: function () {
        var self = this;
        self.fireEvent('cancel', self);
        self.close();
    },

    /**
     * @private
     */
    handleSuccess: function(response, options) {
        var self = this;
        self.fireEvent('success', self, response, options);
        self.close();
    },

    /**
     * @private
     */
    handleFailure: function(response, options) {
        var self = this;
        self.fireEvent('failure', self, response, options);
        self.close();
    }
});
