/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository summary details.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.SummaryView', {
    extend: 'Ext.Panel',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.prefs.DateFormat',
        'Nexus.staging.Icons',
        'Nexus.staging.repository.RepositoriesMediator'
    ],

    requirejs: [
        'Nexus/messages',

        // FIXME: Use requires 'Nexus.ext.LinkButton' once that class is in properly named file/path
        'Nexus/ext/linkbutton'
    ],

    /**
     * The currently selected repository data.
     *
     * @private
     */
    current: undefined,

    /**
     * @override
     */
    initComponent: function () {
        var self = this;

        self.templatePanel = NX.create('Ext.Panel', {
            cls: 'nx-staging-repository-SummaryView-template',
            border: false
        });

        self.descriptionPanel = NX.create('Ext.form.FormPanel', {
            cls: 'nx-staging-repository-SummaryView-description',
            border: false,
            hideLabels: true,

            items: [
                {
                    name: 'description',
                    xtype: 'textarea',
                    height: 50,

                    // If 100% his gets too big and helpText icon rendered below
                    //anchor: '90%',

                    // Right edge can corrupt when using anchor and browser window resizes (textarea box grows, but does not shrink)
                    // so use a specific size, this is a little bigger than what is shown in the confirm dialog
                    width: 400,

                    helpText: 'An optional comment to describe the contents of the staging repository.'

                    // NOTE: Disabling this, I don't really care for how it looks ATM
                    //emptyText: 'Optional description or comment'
                }
            ],

            buttonAlign: 'left',
            buttons: [
                {
                    text: 'Save',
                    formBind: true,
                    scope: self,
                    handler: self.updateDescription
                },
                {
                    xtype: 'link-button',
                    text: 'Discard',
                    formBind: false,
                    scope: self,
                    handler: function() {
                        // reset to last known data
                        self.descriptionPanel.getForm().setValues(self.current);
                    }
                }
            ]
        });

        Ext.apply(self, {
            cls: 'nx-staging-repository-SummaryView',
            title: 'Summary',
            border: false,
            autoScroll: true,

            items: [
                self.templatePanel,
                {
                    xtype: 'fieldset',
                    title: 'Description',

                    items: [
                        self.descriptionPanel
                    ]
                }
            ]
        });

        self.initTemplate();

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * @private
     */
    initTemplate: function() {
        var self = this,
            icons = Nexus.staging.Icons;
        self.mainTpl = NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-SummaryView-body">',
                '{[ this.status(values) ]}',
                '{[ this.properties(values) ]}',
                '{[ this.message(values) ]}',
            '</div>',
            {
                compiled: true,

                link: function(url) {
                    return '<a href="' + url + '" target="_blank">' + url + '</a>';
                },

                timestamp: function(date) {
                    if (date) {
                        // Full date and time
                        return date.format(Nexus.prefs.DateFormat.forName('datetime')['long']);
                    }
                    return undefined;
                },

                status: function(values) {
                    return self.statusTpl.apply(values);
                },

                repository: function (values) {
                    return values.repositoryId + ' (' + values.profileName + ')';
                },

                owner: function (values) {
                    if (values.userId) {
                        if (values.ipAddress) {
                            return values.userId + ' (' + values.ipAddress + ')';
                        }
                        else {
                            return values.userId;
                        }
                    }
                    else {
                        return 'unknown';
                    }
                },

                activity: function(values) {
                    return self.activityTpl.apply(values);
                },

                properties: function(values) {
                    var properties;

                    properties = [
                        { name: 'Repository', value: this.repository(values) },
                        { name: 'Created', value: this.timestamp(values.created) },
                        { name: 'Updated', value: this.timestamp(values.updated) }
                    ];

                    // open and released repos hide the URL
                    if (values.type !== 'open' && values.type !== 'released') {
                        properties.push(
                            // only show URL when the repository has been closed (or its a 404)
                            { name: 'URL', value: this.link(values.repositoryURI) }
                        );
                    }

                    properties.push(
                        { name : 'Activity', value : this.activity(values) },
                        { name : 'Owner', value : this.owner(values) }
                    );

                    if (values.userAgent) {
                        properties.push(
                            { name: 'User-Agent', value: values.userAgent }
                        )
                    }

                    //fire event allowing dynamic customization of properties.
                    Nexus.staging.repository.RepositoriesMediator.fireEvent('staging.repositorySummary.configureProperties', properties, values);

                    return self.propertiesTpl.apply({
                        properties: properties
                    })
                },

                message: function(values) {
                    if (values.type === 'released') {
                        // whos your fuzzy wusszy, ya does that make you feel all nice and comfy inside, yaaaa
                        return self.messageTpl.apply({
                            icon: icons.get('star').img,
                            html: 'Staging repository has been released and can be <b>safely dropped</b>.'
                        });
                    }
                    return '';
                }
            });

        self.activityTpl = NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-SummaryView-activity">',
                '<span class="nx-staging-repository-SummaryView-activity-icon">',
                    '{[ this.icon(values) ]}',
                '</span>',
                '<span class="nx-staging-repository-SummaryView-activity-message">',
                    '{[ this.message(values) ]}',
                '</span>',
            '</div>',
            {
                compiled: true,

                icon: function(values) {
                    if (values.transitioning) {
                        return icons.get('transitioning').img;
                    }
                    else {
                        return icons.forNotificationCount(values.notifications).img;
                    }
                },

                message: function(values) {
                    if (values.transitioning) {
                        return 'Operation in progress';
                    }
                    else {
                        if (values.notifications === 0) {
                          return 'Last operation completed successfully';
                        }
                        else {
                          return 'Last operation failed with ' +
                              Ext.util.Format.plural(values.notifications, 'notification', 'notifications');
                        }
                    }
                }
            });

        self.statusTpl = NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-SummaryView-status">',
                '<div class="nx-staging-repository-SummaryView-status-icon">',
                    '{[ this.statusIcon(values) ]}',
                '</div>',
                '<div class="nx-staging-repository-SummaryView-status-label">',
                    '{[ this.statusLabel(values) ]}',
                '</div>',
            '</div>',
            {
                compiled: true,

                statusIcon: function(values) {
                    return icons.forRepository(values).variant('x32').img;
                },

                statusLabel: function(values) {
                    return values.type;
                }
            });

        self.propertiesTpl = NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-SummaryView-properties">',
                '<table>',
                    '<tpl for="properties">',
                        '<tr class="nx-staging-repository-SummaryView-properties-entry">',
                            '<td class="nx-staging-repository-SummaryView-properties-name">{name}</td>',
                            '<td class="nx-staging-repository-SummaryView-properties-value">{value}</td>',
                        '</tr>',
                    '</tpl>',
                '</table>',
            '</div>',
            {
                compiled: true
            });

        self.messageTpl = NX.create('Ext.XTemplate',
            '<div class="nx-staging-repository-SummaryView-message">',
                '<span>{icon}{html}</span>',
            '</div>',
            {
                compiled: true
            });
    },

    // TDOO: Move updateDescription to Mediator component

    /**
     * Update the description of the selected staging repository.
     *
     * @private
     */
    updateDescription: function() {
        var self = this,
            mediator = Nexus.staging.repository.RepositoriesMediator,
            repo = self.current;

        repo.description = self.descriptionPanel.getForm().findField('description').getValue();

        self.logDebug('Updating repository:', repo.repositoryId, 'description:', repo.description);

        Ext.Ajax.request({
            method: 'PUT',
            url: Sonatype.config.servicePath + '/staging/repository/' + repo.repositoryId,

            jsonData: {
                // for now only send the data back which is relevant
                repositoryId: repo.repositoryId,
                description: repo.description
            },

            scope: self,
            success: function (response, options) {
                mediator.repositoryStore.reload();

                Nexus.messages.show('Staging', 'Description saved');
            },

            failure: function (response, options) {
                self.logError('Failed to save description', response);
            }
        });
    },

    /**
     * Update details.
     *
     * @param repo
     */
    updateRecord: function (repo) {
        var self = this;

        self.current = repo;

        self.mainTpl.overwrite(self.templatePanel.body, repo);

        self.descriptionPanel.getForm().setValues(repo);

        // disable the description panel when when the repository is in transition we don't want to allow changes to it while this is happening
        // also disable the description panel when released, its pretty immutable now
        var disable = repo.transitioning === true || repo.type === 'released';

        // When disabling the entire panel it looks really horrible, so only flip off the textarea and buttons
        //self.descriptionPanel.setDisabled(disable);

        // items is a MixedCollection, use its each()
        self.descriptionPanel.items.each(function(item) {
            item.setDisabled(disable);
        });

        // buttons is an array use Ext.each()
        Ext.each(self.descriptionPanel.buttons, function(item) {
            // hide buttons don't disable
            item.setVisible(!disable);
        });
    }
});
