/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository activity TreeLoader.
 *
 * @since staging 2.3
 *
 * @see java:com.sonatype.nexus.staging.api.dto.StagingActivityDTO
 */
NX.define('Nexus.staging.repository.activity.ActivityTreeLoader', {
    extend: 'Nexus.util.TreeLoaderSupport',

    requires: [
        'Nexus.staging.repository.activity.DetailManager',
        'Nexus.staging.repository.activity.AsyncNodeSupport'
    ],

    /**
     * Template for dataUrl value.  Rendered with node.attribute.data.
     *
     * @property
     */
    URL_TEMPLATE: NX.create('Ext.Template',
        Sonatype.config.servicePath + '/staging/repository/{repositoryId}/activity',
        {
            compiled: true,
            disableFormats: true
        }),

    //
    // FIXME: We are passing &node=<node.id> parameter which is pointless, should sort out how to remove that.
    // FIXME: and/or figure out how to use it so that its functional.
    //

    /**
     * @constructor
     */
    constructor: function (config) {
        var self = this,
            config = config || {};

        Ext.apply(config, {
            requestMethod: 'GET',
            suppressStatus: [ 404 ],
            dataUrl: 'dynamic'
        });

        self.constructor.superclass.constructor.call(self, config);

        // trigger loading data, url will be updated when root node changes
        self.on('beforeload', function(loader, node, callback) {
            self.updateDataUrl(node);
        });

        // on 404 append the empty node
        self.on('loadexception', function(loader, node, response) {
            // HACK: ignore if node already destroyed
            if (!node.ui) {
                self.logWarn('Aborting (empty) node creation; node destroyed:', node);
                return;
            }

            if (response.status === 404) {
                node.appendChild(self.createEmptyNode());
            }
        });
    },

    /**
     * Change the dataUrl for the given node, which is expected to be the root node only.
     *
     * @private
     */
    updateDataUrl: function (node) {
        var self = this,
            data = node.attributes.data;

        NX.assert(data, 'Node missing attribute: data');
        self.dataUrl = self.URL_TEMPLATE.apply(data);
        self.logDebug('Data URL:', self.dataUrl);
    },

    /**
     * @protected
     * @override
     */
    createNodes: function(node, data) {
        var self = this;

        self.logGroup('Creating nodes; parent:', node.text, node);

        for (var i = 0, len = data.length; i < len; i++) {
            // HACK: Abort if the node was destroyed already
            if (!node.ui) {
                self.logWarn('Aborting node creation; parent node destroyed:', node);
                break;
            }

            // check if this is the last node or not
            var last = ((i + 1) === len);

            var child = self.createActivityNode(data[i], last);
            if (child) {
                node.appendChild(child);
            }
        }

        self.logGroupEnd();
    },

    /**
     * Translate an object field (in ISO 8601 format) to a date.
     *
     * @private
     */
    translateDate: function(object, fieldName) {
        var value = object[fieldName];
        if (value) {
            object[fieldName] = Date.parseDate(value, 'c');
        }
    },

    /**
     * Create an activity node.
     *
     * @param activity
     * @param last      True if this is last/most-recent activity.
     *
     * @private
     */
    createActivityNode: function (activity, last) {
        var self = this,
            node,
            hasEvents,
            i,
            child,
            detailProvider;

        hasEvents = Ext.isArray(activity.events) && activity.events.length > 0;

        // transform string dates into objects
        self.translateDate(activity, 'started');
        self.translateDate(activity, 'stopped');

        detailProvider = Nexus.staging.repository.activity.DetailManager.forActivity(activity);

        node = self.createNode({
            text: detailProvider.getTitle(),
            iconCls: detailProvider.getIcon().cls,
            nodeType: 'node',
            leaf: !hasEvents,
            expanded: last,
            dataType: 'activity',
            data: activity
        });

        if (hasEvents) {
            for (i = 0; i < activity.events.length; i++) {
                child = self.createEventNode(activity.events[i]);
                node.appendChild(child);
            }
        }

        return node;
    },

    /**
     * Create an event node.
     *
     * @param event
     *
     * @private
     */
    createEventNode: function (event) {
        var self = this,
            detailProvider;

        // transform string dates into objects
        self.translateDate(event, 'timestamp');

        detailProvider = Nexus.staging.repository.activity.DetailManager.forEvent(event);

        return self.createNode({
            text: detailProvider.getTitle(),
            iconCls: detailProvider.getIcon().cls,
            nodeType: 'node',
            leaf: true,
            dataType: 'event',
            data: event
        });
    },

    /**
     * Create the root node.
     *
     * @return {*}
     */
    createRootNode: function () {
        return NX.create('Ext.tree.TreeNode', {
            text: 'root',
            expanded: true,
            dataType: 'root',
            data: undefined
        });
    },

    /**
     * Create a node for a repository.
     *
     * @param repo
     * @return {*}
     */
    createRepositoryNode: function (repo) {
        var text = 'Activities';

        // show the repository id when debug is enabled
        if (window.location.search === '?debug') {
            text = text + ': ' + repo.repositoryId;
        }

        // HACK: Using custom async node impl so we can cope with aborting load better when node is destroyed
        // HACK: This happens if user changes repository selection w/ Activity panel active before activity
        // HACK: ... nodes have finished loading for the previously selected repository

        var node = NX.create('Nexus.staging.repository.activity.AsyncNodeSupport', {
            text: text,
            expanded: true,
            dataType: 'repository',
            data: repo
        });

        // disallow node collapse
        node.on('beforecollapse', function() { return false; })

        return node;
    },

    /**
     * Create "empty" node which is displayed as only child of repository node when there is no activity data.
     *
     * @return {*}
     */
    createEmptyNode: function() {
        return NX.create('Ext.tree.TreeNode', {
            text: 'No activity recorded',
            iconCls: Nexus.staging.Icons.get('warning').cls,
            nodeType: 'node',
            leaf: true,
            expanded: true,
            dataType: 'empty',
            data: undefined
        });
    }
});
