/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Support to render event details into the message panel.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.activity.EventDetail', {
    extend: 'Nexus.staging.repository.activity.DetailSupport',

    requires: [
        'Nexus.staging.repository.RepositoriesMediator'
    ],

    statics: {
        /**
         * @protected
         */
        headerTpl: NX.create('Ext.XTemplate',
            '{[ this.header(values) ]}',
            {
                compiled: true,

                header: function (values) {
                    var event = values.data,
                        detail = values.detail;

                    // delegate to default detail header template
                    return Nexus.staging.repository.activity.DetailSupport.headerTpl.apply({
                        title: 'Event: ' + detail.getTitle(),
                        icon: detail.getIcon().variant('x32').img,
                        timestamp: Nexus.staging.repository.activity.DetailSupport.timestamp(event.timestamp)
                    });
                }
            }
        ),

        /**
         * Default body template.
         *
         * @protected
         */
        bodyTmpl: NX.create('Ext.XTemplate',
            '{[ this.properties(values) ]}',
            {
                compiled: true,

                properties: function (values) {
                    var event = values.data;

                    return Nexus.staging.repository.activity.DetailSupport.propertiesTpl.apply({
                        properties: event.properties
                    })
                }
            }
        ),

        /**
         * @private
         */
        bodyTplFactories: {
            // HACK: This is here only for testing
            'ruleEvaluate': [
                function(event, detail) {
                    switch (detail.getPropertyValue('typeId')) {
                        case 'testing.always-pass':
                        case 'testing.always-fail':
                            return NX.create('Ext.XTemplate',
                                '<div align="center">',
                                    '<b>OINK?</b>',
                                '</div>',
                                {
                                    compiled: true
                                }
                            );

                        default:
                            return undefined;
                    }
                }
            ],

            // HACK: This is here only for testing
            'rulePassed': [
                function(event, detail) {
                    switch (detail.getPropertyValue('typeId')) {
                        case 'testing.always-pass':
                            return NX.create('Ext.XTemplate',
                                '<div align="center">',
                                    '<b>OINK!</b>',
                                    '<br/><br/>',
                                    '<img src="', Sonatype.config.resourcePath, '/static/images/pig.png">',
                                '</div>',
                                {
                                    compiled: true
                                }
                            );

                        default:
                            return undefined;
                    }
                }
            ],

            // HACK: This is here only for testing
            'ruleFailed': [
                function(event, detail) {
                    switch (detail.getPropertyValue('typeId')) {
                        case 'testing.always-fail':
                            return NX.create('Ext.XTemplate',
                                '<div align="center">',
                                    '<b>DINNER!</b>',
                                    '<br/><br/>',
                                    '<img src="', Sonatype.config.resourcePath, '/static/images/ham.png">',
                                '</div>',
                                {
                                    compiled: true
                                }
                            );

                        default:
                            return undefined;
                    }
                }
            ]
        },

        /**
         * Register a body template factory for an event.
         *
         * @public
         *
         * @param {string} eventName
         * @param {function} factoryFn
         */
        registerBodyTemplateFactory: function(eventName, factoryFn) {
            var factories = Nexus.staging.repository.activity.EventDetail.bodyTplFactories[eventName];
            if (factories === undefined) {
                factories = [];
            }
            factories.push(factoryFn);
        },

        /**
         * @private
         */
        getBodyTemplateFactories: function(event) {
            var factories = Nexus.staging.repository.activity.EventDetail.bodyTplFactories[event.name];
            if (Ext.isArray(factories)) {
                return factories;
            }
            else if (Ext.isFunction(factories)) {
                return [ factories ];
            }
            else {
                NX.assert(factories === undefined, 'Invalid bodyTplFactories entry for event name:', event.name, factories);
            }
            return undefined;
        }
    },

    /**
     * @property
     */
    event: undefined,

    /**
     * @constructor
     */
    constructor: function (event) {
        NX.assert(event, 'Missing event');
        this.event = event;
    },

    /**
     * @override
     */
    getIcon: function () {
        try {
            return Nexus.staging.Icons.forEventName(this.event.name);
        }
        catch (e) {
            this.logError('Missing icon for event: ' + this.event.name);
            return Nexus.staging.Icons.get('bug');
        }
    },

    /**
     * @override
     */
    getTitle: function () {
        return this.event.name;
    },

    /**
     * @private
     */
    selectBodyTemplate: function() {
        var self = this,
            tpl;

        var factories = Nexus.staging.repository.activity.EventDetail.getBodyTemplateFactories(self.event);

        // if there are factories, then ask them to create the template, first one wins
        if (factories) {
            var i;
            for (i = 0; i<factories.length; i++) {
                tpl = factories[i](self.event, self);

                // if the factory created a template, then use it
                if (tpl) {
                    break;
                }
            }
        }

        // use the default if not configured yet
        if (!tpl) {
            tpl = Nexus.staging.repository.activity.EventDetail.bodyTmpl;
        }

        self.logDebug('Selected body template:', tpl);

        return tpl;
    },

    /**
     * @override
     */
    getTemplate: function () {
        var self = this,
            bodyTpl = self.selectBodyTemplate();

        // no need to cache here, we get a new instance each time we render the view
        return NX.create('Ext.XTemplate',
            '<div>',
                '{[ this.header(values) ]}',
                '<div class="nx-staging-repository-ActivityView-messagePanel-body">',
                    '{[ this.body(values) ]}',
                '</div>',
            '</div>',
            {
                //compiled: true,

                header: function (values) {
                    return Nexus.staging.repository.activity.EventDetail.headerTpl.apply(values);
                },

                body: function (values) {
                    return bodyTpl.apply(values);
                }
            }
        );
    },

    // TODO: Split out the properties bits here into a class, this impl is getting messy

    /**
     * @private
     */
    cachedProperties: undefined,

    /**
     * Returns an object representing event properties.
     *
     * @return {*}
     *
     * @protected
     */
    getProperties: function() {
        var self = this,
            result = {},
            value;

        if (self.cachedProperties !== undefined) {
            return self.cachedProperties;
        }

        if (Ext.isArray(self.event.properties)) {
            Ext.each(self.event.properties, function(property) {
                value = self.readPropertyValues(property.name);

                // If there is only one value, then strip off the array
                if (Ext.isArray(value) && value.length === 1) {
                    value = value[0];
                }

                result[property.name] = value;
            });
        }
        else {
            self.logWarn('Event has no properties');
        }

        self.cachedProperties = result;
        //self.logDebug('Cached properties');

        return result;
    },

    /**
     * Read property values directly out of the event.
     *
     * @private
     */
    readPropertyValues: function(name) {
        var self = this,
            result = [];

        if (Ext.isArray(self.event.properties)) {
            Ext.each(self.event.properties, function(property) {
                if (name === property.name) {
                    result.push(property.value);
                }
            });
        }
        else {
            self.logWarn('Event has no properties');
        }

        //if (result.length === 0) {
        //    self.logWarn('Event has no property named:', name);
        //}

        return result;
    },

    /**
     * Check if the even has a named property or not.
     *
     * @protected
     */
    hasProperty: function(name) {
        var self = this,
            value = self.getProperties()[name];

        return value !== undefined && value.length !== 0;
    },

    /**
     * Get named property values.
     *
     * @return {Array} Never undefined.
     *
     * @protected
     */
    getPropertyValues: function(name) {
        var self = this,
            value = self.getProperties()[name];

        if (value === undefined) {
            //self.logWarn('No such property:', name);
            return [];
        }

        // translate to Array if needed
        if (!Ext.isArray(value)) {
            return [ value ];
        }
        return value;
    },

    /**
     * Get a single named property value.
     *
     * @return {String} Never undefined.
     *
     * @protected
     */
    getPropertyValue: function(name) {
        return this.getPropertyValues(name)[0];
    },

    /**
     * Get the name for a rule-set id.
     *
     * @protected
     */
    getRuleSetName: function(ruleSetId) {
        // HACK: special handling for this special rule-set which is always run, its not included in the store
        if (ruleSetId === 'nx-internal-ruleset') {
            return 'Default Always Run';
        }

        return Nexus.staging.repository.RepositoriesMediator.ruleSetStore.nameForId(ruleSetId);
    },

    /**
     * Get the name for a rule-type id.
     *
     * @protected
     */
    getRuleTypeName: function(ruleTypeId) {
        // HACK: special handling for rule included by default, it is not included in the store
        if (ruleTypeId === 'RepositoryWritePolicy') {
            return 'Repository Writable';
        }
        return Nexus.staging.repository.RepositoriesMediator.ruleTypeStore.nameForId(ruleTypeId);
    }
});
