/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,Ext,Sonatype,Nexus*/
/**
 * This is the main configuration container. This panel will hold a formpanel
 * which in turn holds the tab panel for all the configuration
 */
NX.define('Sonatype.repoServer.LdapConfiguration', {
  extend : 'Ext.Panel',
  requires : [
    'Sonatype.repoServer.LdapConfigurationConnection',
    'Sonatype.repoServer.LdapConfigurationBackupMirror',
    'Sonatype.repoServer.LdapConfigurationUserGroup',
    'Sonatype.repoServer.LdapConfigurationSSLCertificate'
  ],

  constructor : function(cfg) {
    var config = cfg || {}, self = this;
    Ext.apply(this, config, {});

    var FIELD_WIDTH = 400;

    this.connectionConfigTab = new Sonatype.repoServer.LdapConfigurationConnection({
      title : 'Connection',
      fieldWidth : FIELD_WIDTH,
      ldapConfigurationContainer : this
    });

    this.mirrorConfigTab = new Sonatype.repoServer.LdapConfigurationBackupMirror({
      title : 'Backup Mirror',
      fieldWidth : FIELD_WIDTH,
      ldapConfigurationContainer : this
    });

    this.userGroupConfigTab = new Sonatype.repoServer.LdapConfigurationUserGroup({
      title : 'User & Group Settings',
      fieldWidth : FIELD_WIDTH,
      ldapConfigurationContainer : this
    });

    this.sslTab = NX.create('Sonatype.repoServer.LdapConfigurationSSLCertificate', {
      title : 'SSL',
      fieldWidth : FIELD_WIDTH,
      ldapConfigurationContainer : this
    });

    this.tabPanel = new Ext.TabPanel({
      activeTab : 0,
      layoutOnTabChange : true,
      border : false,
      header : false,
      anchor : '100% 100%',
      deferredRender : false,
      defaults : {
        layout : 'form',
        labelWidth : 200,
        // NOTE hideMode offsets is required so that the tabs are loaded
        // properly
        hideMode : 'offsets',
        frame : true,
        border : false,
        autoScroll : true
      },
      items : [this.connectionConfigTab, this.mirrorConfigTab, this.userGroupConfigTab, this.sslTab],
      listeners : {
        // on tab change we let the child tabs handle enabling/disabling
        // buttons
        tabchange : {
          fn : function(tabpanel, childpanel) {
            childpanel.handleButtons();
          },
          scope : this
        }
      }
    });

    if (this.id.startsWith('new_') || !this.record.get('ldapUrl').startsWith('ldaps:')) {
      this.tabPanel.on('afterrender', function() {
        Ext.defer(self.tabPanel.hideTabStripItem, 100, self.tabPanel, [self.sslTab]);
      }, {single:true});
    }

    // the layout of the json data sent to server on POST and PUT
    this.referenceData = {
      name : '',
      connectionInfo : {
        searchBase : '',
        systemUsername : '',
        systemPassword : '',
        authScheme : '',
        protocol : '',
        host : '',
        port : 0,
        backupMirrorProtocol : '',
        backupMirrorHost : '',
        backupMirrorPort : 0,
        realm : '',
        connectionTimeout : 0,
        connectionRetryDelay : 0,
        cacheTimeout : 0
      },
      userAndGroupConfig : {
        emailAddressAttribute : '',
        ldapGroupsAsRoles : false,
        groupBaseDn : '',
        groupIdAttribute : '',
        groupMemberAttribute : '',
        groupMemberFormat : '',
        groupObjectClass : '',
        userPasswordAttribute : '',
        userIdAttribute : '',
        userObjectClass : '',
        ldapFilter : '',
        userBaseDn : '',
        userRealNameAttribute : '',
        userSubtree : false,
        groupSubtree : false,
        userMemberOfAttribute : ''
      }
    };

    // the main form panel, and the buttons tied to it
    this.formPanel = new Ext.form.FormPanel({
      border : false,
      frame : true,
      items : [this.tabPanel],
      buttonAlign : 'center',
      buttons : [
        {
          text : 'Check Authentication',
          handler : this.checkAuthenticationHandler,
          scope : this,
          hidden : true,
          disabled : !this.sp.checkPermission('security:ldapconfig', this.sp.EDIT)
        },
        {
          text : 'Check User Mapping',
          handler : this.checkUserMappingHandler,
          scope : this,
          hidden : true,
          disabled : !this.sp.checkPermission('security:ldapconfig', this.sp.EDIT)
        },
        {
          text : 'Check Login',
          handler : this.checkLoginHandler,
          scope : this,
          hidden : true,
          disabled : !this.sp.checkPermission('security:ldapconfig', this.sp.EDIT)
        },
        {
          text : 'Save',
          handler : this.saveHandler,
          scope : this,
          disabled : !this.sp.checkPermission('security:ldapconfig', this.sp.CREATE)
        },
        {
          text : 'Cancel',
          handler : this.cancelHandler,
          scope : this
        }
      ]
    });

    Sonatype.repoServer.LdapConfiguration.superclass.constructor.call(this, {
      layout : 'fit',
      border : false,
      items : [this.formPanel],
      listeners : {
        afterlayout : {
          fn : this.loadHandler,
          scope : this
        }
      }
    });
  },
  // check if all child tabs are loaded, the idea here is that we can't
  // load the form data until are child tabs are layed out, if the fields
  // dont yet exist, the form load won't populate them
  allTabsLoaded : function() {
    if (this.connectionConfigTab.isLoaded && this.mirrorConfigTab.isLoaded && this.userGroupConfigTab.isLoaded) {
      return true;
    }

    return false;
  },
  // force each of the child tabs to layout
  layoutTabs : function() {
    this.connectionConfigTab.doLayout();
    this.mirrorConfigTab.doLayout();
    this.userGroupConfigTab.doLayout();
  },
  // load data from the server, this will be called on Add of new server as
  // well
  // but in that case won't do anything
  loadHandler : function() {
    // only load once..
    if (this.isLoaded) {
      return;
    }

    // we only want to do these items once, not on each deferred call
    if (!this.isLoading) {
      this.isLoading = true;
      this.layoutTabs();

      // no request to server on new item, so dont show window
      if (!this.isNew) {
        Sonatype.MessageBox.wait('Loading LDAP Configuration...');
      }
    }

    // we defer call to this method if tabs aren't loaded
    if (!this.allTabsLoaded()) {
      this.loadHandler.defer(100, this);
      return;
    }

    if (!this.isNew) {
      this.formPanel.getForm().doAction('sonatypeLoad', {
        url : this.record.get('url'),
        method : 'GET',
        fpanel : this,
        scope : this,
        dataModifiers: this.dataModifiers.load,
        success : function(form, action) {
          // do special logic if necessary, i.e. need to expand/collapse
          // the group config
          this.connectionConfigTab.postLoad();
          this.mirrorConfigTab.postLoad();
          this.userGroupConfigTab.postLoad();
          this.isLoaded = true;
          Sonatype.MessageBox.hide();
        },
        failure : function(form, action) {
          Sonatype.utils.connectionError(action.response, 'Error Loading LDAP Configuration.');
        }
      });
    }
    else {
      this.connectionConfigTab.postLoad();
      this.mirrorConfigTab.postLoad();
      this.userGroupConfigTab.postLoad();
    }
  },
  // save the data to the server, once save is complete, call container so
  // grid can
  // be updated, if validation fails, select the invalid tab
  saveHandler : function() {
    if (this.formPanel.getForm().isValid()) {
      this.formPanel.getForm().doAction('sonatypeSubmit', {
        method : this.isNew ? 'POST' : 'PUT',
        url : this.isNew ? Sonatype.config.servicePath + '/ldap/servers' : this.record.get('url'),
        waitMsg : this.isNew ? 'Creating LDAP Server...' : 'Updating LDAP Server...',
        fpanel : this.formPanel,
        dataModifiers: this.dataModifiers.submit,
        serviceDataObj : this.referenceData,
        isNew : this.isNew,
        scope : this,
        success : function(form, action) {
          if (this.ldapSaveCompleteHandler) {
            this.ldapSaveCompleteHandler.call(this.ldapContainer, this, Ext.decode(action.response.responseText));
          }
          this.isNew = false;
          if ( this.record.get('ldapUrl').startsWith('ldaps') ) {
            this.tabPanel.unhideTabStripItem(this.sslTab);
          } else {
            this.tabPanel.hideTabStripItem(this.sslTab);
          }
        },
        failure : function(form, action) {
          Sonatype.utils.connectionError(action.response, 'Error Saving LDAP Configuration.', false, {
            hideErrorStatus : true,
            decodeErrorResponse : true
          });
          // set result to null so the dialog pops up.
          action.result.errors = null;
        }
      });
    }
    else {
      this.selectInvalidTab();
    }
  },
  // cancel the configuration, will simply remove the card panel
  cancelHandler : function() {
    if (this.ldapCancelHandler) {
      this.ldapCancelHandler.call(this.ldapContainer, this);
    }
  },
  // pass form data to server to check for authentication success/failure
  checkAuthenticationHandler : function() {
    var panel = this.tabPanel.getActiveTab();

    if (panel.getId() == this.connectionConfigTab.getId()) {
      if (this.isPanelValid(this.connectionConfigTab)) {
        this.connectionConfigTab.checkAuthentication();
      }
    }
    else if (panel.getId() == this.mirrorConfigTab.getId()) {
      if (!this.isPanelValid(this.connectionConfigTab)) {
        this.tabPanel.setActiveTab(this.connectionConfigTab);
      }
      else if (this.isPanelValid(this.mirrorConfigTab)) {
        this.mirrorConfigTab.checkAuthentication();
      }
    }
  },
  // open up the user mapping panel
  checkUserMappingHandler : function() {
    if (this.formPanel.getForm().isValid()) {
      this.userGroupConfigTab.checkUserMapping(this.formPanel, this.referenceData);
    }
    else {
      this.selectInvalidTab();
    }
  },
  checkLoginHandler : function() {
    if (this.formPanel.getForm().isValid()) {
      this.userGroupConfigTab.checkLogin(this.formPanel, this.referenceData);
    }
    else {
      this.selectInvalidTab();
    }
  },
  // select an invalid tab if form validation fails
  // if the currently active panel has validation errors, select that
  // otherwise find the one that does and select it
  selectInvalidTab : function() {
    var panel = this.tabPanel.getActiveTab();

    // active tab isn't valid, no need to switch
    if (!this.isPanelValid(panel)) {
      return;
    }

    if (panel.getId() != this.connectionConfigTab.getId() && !this.isPanelValid(this.connectionConfigTab)) {
      this.tabPanel.setActiveTab(this.connectionConfigTab);
    }
    else if (panel.getId() != this.mirrorConfigTab.getId() && !this.isPanelValid(this.mirrorConfigTab)) {
      this.tabPanel.setActiveTab(this.mirrorConfigTab);
    }
    else if (panel.getId() != this.userGroupConfigTab.getId() && !this.isPanelValid(this.userGroupConfigTab)) {
      this.tabPanel.setActiveTab(this.userGroupConfigTab);
    }
  },
  // check if form fields on the panel are valid
  isPanelValid : function(panel) {
    var items = panel.items.getRange();
    for (var i = 0; i < items.length; i++) {
      if (items[i].xtype == 'fieldset' && !this.isPanelValid(items[i])) {
        return false;
      }
      else if (items[i].isValid && !items[i].isValid()) {
        return false;
      }
    }

    return true;
  },
  expandHandler : function(panel) {
    panel.items.each(function(item, i, len) {
      if (item.getEl() != null && item.getEl().up('div.required-field', 3)) {
        item.allowBlank = false;
        item.clearInvalid();
      }
      else {
        item.allowBlank = true;
      }
    }, this);
  },
  collapseHandler : function(panel) {
    panel.items.each(function(item, i, len) {
      item.allowBlank = true;
    }, this);
  },
  dataModifiers: {
    load: {
      "connectionInfo.systemUsername": function (value) {
        if (Ext.isDefined(value)) {
          return Sonatype.utils.base64.decode(value);
        }
        return value;
      },
      "connectionInfo.systemPassword": function (value) {
        if (Ext.isDefined(value)) {
          return Sonatype.utils.base64.decode(value);
        }
        return value;
      }
    },
    submit: {
      "connectionInfo.systemUsername": function (value) {
        if (Ext.isDefined(value)) {
          return Sonatype.utils.base64.encode(value);
        }
        return value;
      },
      "connectionInfo.systemPassword": function (value) {
        if (Ext.isDefined(value)) {
          return Sonatype.utils.base64.encode(value);
        }
        return value;
      }
    }
  }
});