/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*
 * This is the main container that holds all of the ldap stuff it includes a
 * gridPanel which lists the currently defined ldap servers and a card layout
 * panel which holds the configuration panels for each ldap server
 */
/*global NX,Ext,Sonatype,Nexus*/
NX.define('Sonatype.repoServer.LdapContainer', {
  extend : 'Ext.Panel',
  requires : [
    'Sonatype.repoServer.LdapGrid',
    'Sonatype.repoServer.LdapConfiguration'
  ],
  constructor : function(cfg) {
    var config = cfg || {},
          defaultConfig = {
            title : 'LDAP Configuration'
          };
    Ext.apply(this, config, defaultConfig);

    this.sp = Sonatype.lib.Permissions;

    // the grid, we use the add/delete/rowclick handlers here
    // to manage what is shown in the card panel below
    this.ldapGrid = new Sonatype.repoServer.LdapGrid({
      ldapContainer : this,
      region : 'center',
      ldapAddHandler : this.addHandler,
      ldapDeleteHandler : this.deleteHandler,
      ldapRowclickHandler : this.rowclickHandler,
      sp : this.sp
    });

    // The card panel that holds the configurations for the ldap servers
    this.ldapConfigCards = new Ext.Panel({
      split : true,
      region : 'south',
      height : Sonatype.view.mainTabPanel.getInnerHeight() * 0.75,
      minHeight : 200,
      deferredRender : false,
      autoScroll : false,
      layout : 'card',
      activeItem : 0,
      items : [
        {
          id : 'default',
          border : false,
          html : '<div class="little-padding">Select an LDAP record to view the details.</div>'
        }
      ]
    });

    Sonatype.repoServer.LdapContainer.superclass.constructor.call(this, {
      layout : 'border',
      border : 'false',
      items : [this.ldapGrid, this.ldapConfigCards]
    });
  },

  // when a new ldap server is added, create a new config panel
  // and add to the card list
  addHandler : function(grid, record) {
    var panel = new Sonatype.repoServer.LdapConfiguration({
      ldapContainer : this,
      id : record.id,
      record : record,
      isNew : true,
      ldapSaveCompleteHandler : this.saveCompleteHandler,
      ldapCancelHandler : this.cancelHandler,
      sp : this.sp
    });
    this.ldapConfigCards.add(panel);
    this.ldapConfigCards.getLayout().setActiveItem(panel);
  },
  // when an ldap server is deleted, we need to remove from the
  // cards list
  deleteHandler : function(grid, record) {
    var panel = this.ldapConfigCards.findById(record.id);

    if (panel) {
      this.ldapConfigCards.remove(panel, true);
      panel.destroy();
    }

    this.selectLastCard();
  },
  selectLastCard : function() {
    // set the active item to the last item selected
    var panel = this.ldapConfigCards.items.last();

    this.ldapConfigCards.getLayout().setActiveItem(panel);

    // need to set the selection up in the grid as well
    if (this.ldapConfigCards.items.getCount() > 1) {
      var record = this.ldapGrid.store.getById(panel.id);
      this.ldapGrid.getSelectionModel().selectRecords([record], false);
    }
    else {
      this.ldapGrid.getSelectionModel().clearSelections();
    }
  },
  // When you click on an item in the grid, need to bring up its card panel
  // if already exists, otherwise need to create a new configuration panel
  rowclickHandler : function(grid, record) {
    var panel = this.ldapConfigCards.findById(record.id);

    if (panel) {
      this.ldapConfigCards.getLayout().setActiveItem(panel);
    }
    else {
      panel = new Sonatype.repoServer.LdapConfiguration({
        record : record,
        ldapContainer : this,
        id : record.id,
        ldapSaveCompleteHandler : this.saveCompleteHandler,
        ldapCancelHandler : this.cancelHandler,
        sp : this.sp
      });

      this.ldapConfigCards.add(panel);
      this.ldapConfigCards.getLayout().setActiveItem(panel);
    }
  },
  /**
   * When save is called, on success we need to update the details in
   * the grid record, and update the id of the card panel.
   *
   * The panel is removed first, then re-added as if it was selected by the user.
   * If it was not re-added, there was a mismatch between what the detail panel *thought*
   * it contained and what was actually displayed.
   */
  saveCompleteHandler : function(configurationPanel, jsonResponse) {
    this.ldapConfigCards.remove(configurationPanel, true);

    var id = configurationPanel.id;
    configurationPanel.id = jsonResponse.data.id;

    this.rowclickHandler(this.ldapGrid, this.ldapGrid.updateRecord(id, jsonResponse.data));
  },
  cancelHandler : function(configurationPanel) {
    if (configurationPanel.isNew) {
      this.ldapGrid.deleteHandler();
    }
    else {
      this.ldapConfigCards.remove(configurationPanel, true);
      configurationPanel.destroy();

      this.selectLastCard();
    }
  },
  // simply clear all cards out
  refresh : function() {
    this.ldapConfigCards.getLayout().setActiveItem(0);
    while (this.ldapConfigCards.items.getCount() > 1) {
      var panel = this.ldapConfigCards.items.last();
      this.ldapConfigCards.remove(panel, true);
      panel.destroy();
    }
  }
}, function() {
  // Add the link to left navigation
  Sonatype.Events.addListener('nexusNavigationInit', function(nexusPanel) {
    var sp = Sonatype.lib.Permissions;

    if (sp.checkPermission('security:ldapconfig', sp.READ)) {
      nexusPanel.add({
        enabled : true,
        sectionId : 'st-nexus-security',
        title : 'Enterprise LDAP',
        tabId : 'enterprise-ldap',
        tabCode : Sonatype.repoServer.LdapContainer
      });
    }
  });
});

