/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Sonatype, Nexus, Ext*/

define('Nexus/usertoken', [
  'Nexus/config', // Sonatype.config
  'Nexus/siesta',
  'Nexus/messages'
], function () {

  // FIXME: Replace with singleton class

  Ext.namespace('Nexus.usertoken');

  /**
   * Base identifier for all plugins bits.
   *
   * @type {String}
   * @static
   */
  Nexus.usertoken.ID = 'nx-usertoken';

  /**
   * Title for various bits.
   *
   * @type {String}
   * @static
   */
  Nexus.usertoken.TITLE = 'User Token';

  /**
   * Show a user-token transient message.
   *
   * @param message
   * @static
   */
  Nexus.usertoken.showMessage = function (message) {
    Nexus.messages.show(Nexus.usertoken.TITLE, message);
  };

  /**
   * Returns the path to a user-token REST resource.
   *
   * @param path
   * @return {String}
   * @static
   */
  Nexus.usertoken.resourcePath = function (path) {
    if (!path.startsWith('/')) {
      path = '/' + path;
    }
    return Nexus.siesta.basePath + '/usertoken' + path;
  };

  /**
   * Flag to indicate if user-token features are enabled in the UI or not.
   *
   * @type {Boolean}
   * @static
   */
  Nexus.usertoken.enabled = false;

  /**
   * Modify user-token enabled flag.
   *
   * @param enabled
   * @static
   */
  Nexus.usertoken.setEnabled = function (enabled) {
    Nexus.log('Nexus.usertoken.enabled: ' + enabled);
    Nexus.usertoken.enabled = enabled;
  };

  // Configure event listener to enable/disable user-token when the current user changes
  Sonatype.Events.addListener('nexusStatus', function () {
    if (Sonatype.user.curr && Sonatype.user.curr.isLoggedIn) {
      // If logged in, then request status
      Ext.Ajax.request({
        url: Nexus.usertoken.resourcePath('/status'),
        method: 'GET',
        suppressStatus: true, // suppress all error status

        success: function (response) {
          var status = Ext.decode(response.responseText);
          Nexus.usertoken.setEnabled(status.enabled);
        },

        failure: function () {
          // force back to disabled on any failure
          Nexus.usertoken.setEnabled(false);
        }
      });
    }
    else {
      // if not logged in reset back to false
      Nexus.usertoken.setEnabled(false);
    }
  });

});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/**
 * User-token settings view.
 *
 * @class Nexus.usertoken.SettingsView
 * @extends Ext.Panel
 * @namespace Nexus.usertoken
 */
NX.define('Nexus.usertoken.SettingsView', {
  extend: 'Ext.Panel',

  requires: [
    'Nexus.wonderland.AuthenticateButton',
    'Nexus.usertoken.Icons'
  ],
  requirejs: ['Nexus/usertoken'],
  mixins: [
    'Nexus.LogAwareMixin'
  ],

  constructor: function (config) {
    var self = this,
        icons = Nexus.usertoken.Icons;

    self.protectContentCheckbox = new Ext.form.Checkbox({
      name: 'protectContent',
      fieldLabel: 'Protect Content',
      helpText: 'Select this option to require that user tokens are used for repository authentication. This does not affect UI access.',
      disabled: true
    });

    self.settingsForm = new Ext.FormPanel({
      border: false,
      labelWidth: 100,
      layoutConfig: {
        labelSeparator: ''
      },
      items: [
        {
          xtype: 'checkbox',
          name: 'enabled',
          fieldLabel: 'Enabled',
          helpText: 'Enable user token functionality'
        },
        self.protectContentCheckbox
      ],
      buttons: [
        {
          text: 'Save',
          tooltip: 'Save settings',
          formBind: true,
          scope: self,
          handler: self.saveSettings
        },
        new Nexus.ext.LinkButton({
          text: 'Discard',
          tooltip: 'Discard changes',
          formBind: false,
          scope: self,
          handler: self.discardSettings
        })
      ]
    });

    // TODO: Show warning indicator if security is disabled and/or realm is not configured?

    // TODO: Show warning indicator if capability failed to activate

    self.resetPanel = new Ext.form.FieldSet({
      title: 'Reset',
      autoHeight: true,
      hidden: true,
      items: [
        {
          xtype: 'component',
          cls: 'nx-usertoken-icon',
          html: icons.get('error').variant('x32').img
        },
        {
          xtype: 'label',
          html: 'Reset will invalidate <span style="font-weight: bold;">ALL</span> existing user tokens and force new tokens to be created the next time accessed.<br/><br clear="both"/>',
          cls: 'nx-usertoken-message'
        },
        {
          xtype: 'nx-wonderland-button-authenticate',
          text: 'Reset All User Tokens',
          tooltip: 'Authenticate and reset all user tokens',
          listeners: {
            'authenticated': {
              scope: self,
              fn: self.resetAllUserTokens
            }
          }
        }
      ]
    });

    Ext.apply(self, {
      cls: 'nx-usertoken-SettingsView',
      border: false,
      layout: 'vbox',
      layoutConfig: {
        align: 'stretch'
      },
      items: [
        {
          xtype: 'fieldset',
          title: 'Settings',
          autoHeight: true,
          items: [
            self.settingsForm
          ]
        },

        self.resetPanel
      ]
    });

    // Call super constructor
    self.constructor.superclass.constructor.apply(self, arguments);

    // Load settings after layout, has to be done post superclass.constructor()
    self.on('afterlayout', self.loadSettings, self, {
      single: true
    });
  },

  getWaitMask: function () {
    var self = this;

    if (!self.waitMask) {
      self.waitMask = new Ext.LoadMask(self.getEl(), {
        msg: 'Please wait'
      });
    }
    return self.waitMask;
  },

  /**
   * @param {Boolean} enabled
   */
  setEnabled: function (enabled) {
    // Update enabled flag, this won't really help too much if the view is already opened
    Nexus.usertoken.enabled = enabled;

    // Toggle components visible
    this.resetPanel.setVisible(enabled);

    // Toggle components enabled
    if (enabled) {
      this.protectContentCheckbox.enable();
    }
    else {
      this.protectContentCheckbox.disable();
    }

    // need to re-layout because resetPanel will start hidden and initially has no size
    this.doLayout();
  },

  saveSettings: function () {
    var self = this, form;

    self.logDebug('Save settings');

    self.getWaitMask().show();

    form = self.settingsForm.getForm();

    Ext.Ajax.request({
      url: Nexus.usertoken.resourcePath('/settings'),
      method: 'PUT',
      scope: self,

      jsonData: {
        enabled: form.findField('enabled').getValue(),
        protectContent: form.findField('protectContent').getValue()
      },

      callback: function () {
        self.getWaitMask().hide();
      },

      success: function () {
        Nexus.usertoken.showMessage('Settings saved');
        self.setEnabled(form.findField('enabled').getValue());
      }
    });
  },

  loadSettings: function () {
    var self = this;

    self.logDebug('Load settings');

    self.getWaitMask().show();

    Ext.Ajax.request({
      url: Nexus.usertoken.resourcePath('/settings'),
      method: 'GET',
      scope: self,

      callback: function () {
        self.getWaitMask().hide();
      },

      success: function (response) {
        var settings = Ext.decode(response.responseText);
        self.settingsForm.getForm().setValues({
          enabled: settings.enabled,
          protectContent: settings.protectContent
        });
        self.logDebug('Settings loaded');

        self.setEnabled(settings.enabled);
      }
    });
  },

  discardSettings: function () {
    var self = this;

    self.logDebug('Discard settings');

    // reload settings
    self.loadSettings();
  },

  resetAllUserTokens: function (button, authTicket) {
    var self = this;

    self.logDebug('Reset all user tokens');

    self.getWaitMask().show();

    Ext.Ajax.request({
      url: Nexus.usertoken.resourcePath('/users'),
      method: 'DELETE',
      scope: self,

      headers: {
        'X-NX-AuthTicket': authTicket
      },

      callback: function () {
        self.getWaitMask().hide();
      },

      success: function () {
        Nexus.usertoken.showMessage('All user tokens have been reset');
      }
    });
  }

}, function () {
  var type = this;

  // Attach SettingsView to navigation panel if current user has appropriate permissions
  Sonatype.Events.addListener('nexusNavigationInit', function (panel) {
    var p = Sonatype.lib.Permissions;
    if (p.checkPermission('nexus:usertoken:settings', p.READ)) {
      panel.add({
        enabled: true,
        sectionId: 'st-nexus-security',
        title: Nexus.usertoken.TITLE,
        tabId: 'usertoken',
        tabTitle: Nexus.usertoken.TITLE,
        tabCode: type
      });
    }
  });
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/**
 * Dialog to display a user's user-token details.
 *
 * @class Nexus.usertoken.UserTokenWindow
 * @extends Ext.Window
 * @namespace Nexus.usertoken
 */
NX.define('Nexus.usertoken.UserTokenWindow', {
  extend: 'Ext.Window',

  requires: [
    'Nexus.usertoken.Icons'
  ],
  requirejs: ['Nexus/usertoken'],

  /**
   * @cfg {String} nameCode   The user-token name-code.
   * @cfg {String} passCode   The user-token pass-code.
   */
  constructor: function (config) {
    var self = this,
        icons = Nexus.usertoken.Icons,
        exampleXml;

    Ext.applyIf(config, {
      message: 'You have requested an operation which requires validation of your credentials.'
    });

    exampleXml = '<server>\n' +
        '  <id>${server}</id>\n' +
        '  <username>' + config.nameCode + '</username>\n' +
        '  <password>' + config.passCode + '</password>\n' +
        '</server>';

    // FIXME: Probably should sort out dynamic width of bits based on encoding strategy

    Ext.apply(self, {
      cls: 'nx-usertoken-UserTokenWindow',
      title: Nexus.usertoken.TITLE,
      border: false,
      closable: true,
      modal: true,
      autoHeight: true,
      width: 550,
      resizable: false,
      items: [
        {
          xtype: 'panel',
          items: [
            {
              xtype: 'component',
              cls: 'nx-usertoken-icon',
              html: icons.get('key').variant('x32').img
            },
            {
              xtype: 'label',
              html: 'User tokens are a combination of a name and password codes.<br/>' +
                  '<span style="font-weight: bold;">Keep these codes secret!</span><br/><br clear="both"/>' +
                  'Your user token is:<br/><br/>'
            },
            {
              xtype: 'textfield',
              name: 'nameCode',
              width: 100,
              readOnly: true,
              style: {
                padding: '2px',
                'font-family': 'monospace'
              },
              listeners: {
                render: self.installClickToSelectHandler
              }
            },
            {
              xtype: 'label',
              html: ':',
              style: {
                padding: '2px',
                'font-family': 'monospace'
              }
            },
            {
              xtype: 'textfield',
              name: 'passCode',
              width: 360,
              readOnly: true,
              style: {
                padding: '2px',
                'font-family': 'monospace'
              },
              listeners: {
                render: self.installClickToSelectHandler
              }
            },
            {
              xtype: 'label',
              html: '<br/><br/>Use the following in your Maven settings.xml:<br/><br/>'
            },
            {
              xtype: 'textarea',
              name: 'exampleXml',
              height: 90,
              width: 510,
              readOnly: true,
              style: {
                padding: '2px',
                'font-family': 'monospace'
              },
              listeners: {
                render: self.installClickToSelectHandler
              }
            },
            {
              xtype: 'label',
              html: '<br/><br/>This window will automatically close after one minute.'
            }
          ],
          buttons: [
            {
              text: 'Close',
              scope: self,
              handler: function () {
                self.close();
              }
            }
          ]
        }
      ],
      listeners: {
        show: function (component) {
          component.find('name', 'nameCode')[0].setValue(config.nameCode);
          component.find('name', 'passCode')[0].setValue(config.passCode);
          component.find('name', 'exampleXml')[0].setValue(exampleXml);

          // Automatically close the window
          (function () {
            if (self.isVisible()) { // ignore if already closed
              Nexus.usertoken.showMessage('Automatically closing window due to timeout');
              self.close();
            }
          }).defer(1 * 60 * 1000); // 1 minute
        }
      }
    });

    // Call super constructor
    self.constructor.superclass.constructor.apply(self, arguments);
  },

  /**
   * Installs click handler to automatically select component text.
   */
  installClickToSelectHandler: function (component) {
    component.getEl().on('click', function () {
      component.selectText();
    });
  }
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Container for icons used by usertoken plugin.
 *
 * @since 2.7
 */
NX.define('Nexus.usertoken.Icons', {
  extend: 'Nexus.util.IconContainer',
  singleton: true,

  /**
   * @constructor
   */
  constructor: function () {
    var me = this;

    // helper to build an icon config with variants, where variants live in directories, foo.png x16 -> x16/foo.png
    function iconConfig(fileName, variants) {
      var config = {};
      if (variants === undefined) {
        variants = [ 'x32', 'x16' ]
      }
      Ext.each(variants, function(variant) {
        config[variant] = variant + '/' + fileName;
      });
      return config;
    }

    me.constructor.superclass.constructor.call(me, {
      stylePrefix: 'nx-usertoken-icon-',

      icons: {
        error: iconConfig('error.png'),
        key: iconConfig('key.png')
      }
    });
  }

});
/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/**
 * User-token user-profile view.
 *
 * @class Nexus.usertoken.UserProfileView
 * @extends Ext.Panel
 * @namespace Nexus.usertoken
 */
NX.define('Nexus.usertoken.UserProfileView', {
  extend: 'Ext.Panel',
  requires: [
    'Nexus.wonderland.AuthenticateButton',
    'Nexus.usertoken.UserTokenWindow',
    'Nexus.usertoken.Icons'
  ],
  requirejs: ['Nexus/usertoken'],
  mixins: [
    'Nexus.LogAwareMixin'
  ],

  constructor: function (config) {
    var self = this,
        icons = Nexus.usertoken.Icons;

    Ext.apply(self, {
      cls: 'nx-usertoken-UserProfileView',
      border: false,
      items: [
        {
          xtype: 'component',
          cls: 'nx-usertoken-icon',
          html: icons.get('key').variant('x32').img
        },
        {
          xtype: 'label',
          cls: 'nx-usertoken-message',
          html: 'User tokens provide an alternative mechanism to authenticate with Nexus without use of passwords.<br clear="both"/>'
        },
        {
          xtype: 'fieldset',
          title: 'Access',
          autoHeight: true,
          items: [
            {
              xtype: 'label',
              html: 'A new user token will be created the first time it is accessed.<br/><br clear="both"/>',
              cls: 'nx-usertoken-message'
            },
            {
              xtype: 'nx-wonderland-button-authenticate',
              text: 'Access User Token',
              tooltip: 'Authenticate and access user token',
              listeners: {
                'authenticated': {
                  scope: self,
                  fn: self.accessUserToken
                }
              }
            }
          ]
        },
        {
          xtype: 'fieldset',
          title: 'Reset',
          autoHeight: true,
          items: [
            {
              xtype: 'label',
              html: 'Resetting your user token will invalidate the current token and force a new token to be created the next time accessed.<br/><br/>',
              cls: 'nx-usertoken-message'
            },
            {
              xtype: 'nx-wonderland-button-authenticate',
              text: 'Reset User Token',
              tooltip: 'Authenticate and reset user token',
              listeners: {
                'authenticated': {
                  scope: self,
                  fn: self.resetUserToken
                }
              }
            }
          ]
        }
      ]
    });

    // Call super constructor
    self.constructor.superclass.constructor.apply(self, arguments);
  },

  getWaitMask: function () {
    var self = this;

    if (!self.waitMask) {
      self.waitMask = new Ext.LoadMask(self.getEl(), {
        msg: 'Please wait'
      });
    }
    return self.waitMask;
  },

  accessUserToken: function (button, authTicket) {
    var self = this;

    self.logDebug('Access user token');

    self.getWaitMask().show();

    Ext.Ajax.request({
      url: Nexus.usertoken.resourcePath('/current'),
      method: 'GET',
      scope: self,

      headers: {
        'X-NX-AuthTicket': authTicket
      },

      callback: function () {
        self.getWaitMask().hide();
      },

      success: function (response) {
        var token, window;
        token = Ext.decode(response.responseText);
        window = new Nexus.usertoken.UserTokenWindow({
          nameCode: token.nameCode,
          passCode: token.passCode
        });
        window.show();
      }
    });
  },

  resetUserToken: function (button, authTicket) {
    var self = this;

    self.logDebug('Reset user token');

    self.getWaitMask().show();

    Ext.Ajax.request({
      url: Nexus.usertoken.resourcePath('/current'),
      method: 'DELETE',
      scope: self,

      headers: {
        'X-NX-AuthTicket': authTicket
      },

      callback: function () {
        self.getWaitMask().hide();
      },

      success: function () {
        Nexus.usertoken.showMessage('Your user token has been reset');
      }
    });
  },

  shouldShow: function () {
    var p = Sonatype.lib.Permissions;
    return Nexus.usertoken.enabled && p.checkPermission('nexus:usertoken:current', p.CREATE, p.READ, p.DELETE)
  }

}, function () {
  var type = this;
  Nexus.profile.register(Nexus.usertoken.TITLE, type, ['user']);
});

/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/**
 * User-token administrator user-profile view.
 *
 * @class Nexus.usertoken.AdminProfileView
 * @extends Ext.Panel
 * @namespace Nexus.usertoken
 */
NX.define('Nexus.usertoken.AdminProfileView', {
  extend: 'Ext.Panel',

  requires: [
    'Nexus.wonderland.AuthenticateButton'
  ],
  requirejs: ['Nexus/usertoken'],
  mixins: [
    'Nexus.LogAwareMixin'
  ],

  /**
   * @cfg {String} username
   */
  constructor: function (config) {
    var self = this;

    if (!config.username) {
      throw 'Missing config.username';
    }

    Ext.apply(self, {
      cls: 'nx-usertoken-AdminProfileView',
      border: false,
      items: [
        {
          xtype: 'fieldset',
          title: 'Reset',
          autoHeight: true,
          items: [
            {
              xtype: 'label',
              html: 'Resetting a users token will invalidate the current token and force a new token to be created the next time accessed by the user.<br/><br/>',
              cls: 'nx-usertoken-message'
            },
            {
              xtype: 'nx-wonderland-button-authenticate',
              text: 'Reset User Token',
              tooltip: 'Authenticate and reset this users token',
              listeners: {
                'authenticated': {
                  scope: self,
                  fn: self.resetUserToken
                }
              }
            }
          ]
        }
      ]
    });

    // Call super constructor
    self.constructor.superclass.constructor.apply(self, arguments);
  },

  getWaitMask: function () {
    var self = this;

    if (!self.waitMask) {
      self.waitMask = new Ext.LoadMask(self.getEl(), {
        msg: 'Please wait'
      });
    }
    return self.waitMask;
  },

  resetUserToken: function (button, authTicket) {
    var self = this;

    self.logDebug('Reset user token: ' + self.username);

    self.getWaitMask().show();

    Ext.Ajax.request({
      url: Nexus.usertoken.resourcePath('/users/' + self.username),
      method: 'DELETE',

      headers: {
        'X-NX-AuthTicket': authTicket
      },

      callback: function () {
        self.getWaitMask().hide();
      },

      success: function () {
        Nexus.usertoken.showMessage('User token has been reset for user: ' + self.username);
      }
    });
  },

  shouldShow: function () {
    var p = Sonatype.lib.Permissions;
    return Nexus.usertoken.enabled && p.checkPermission('nexus:usertoken:user', p.READ, p.DELETE);
  }

}, function () {
  var type = this;
  Nexus.profile.register(Nexus.usertoken.TITLE, type, ['admin']);
});


/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/
define('nexus-usertoken-plugin-boot', [
  'Nexus/usertoken',
  'Nexus/usertoken/SettingsView',
  'Nexus/usertoken/AdminProfileView',
  'Nexus/usertoken/UserProfileView'
], function() {
        Nexus.Log.debug('Usertoken plugin loaded');
      }
);

